@@ -1,5 +1,6 @@
 """Certbot command line argument & config processing."""
 # pylint: disable=too-many-lines
+# ruff: noqa: F401
 import argparse
 import logging
 import logging.handlers
@@ -55,7 +56,7 @@ logger = logging.getLogger(__name__)
 helpful_parser: Optional[HelpfulArgumentParser] = None
 
 
-def prepare_and_parse_args(plugins: plugins_disco.PluginsRegistry, args: List[str]
+def prepare_and_parse_args(plugins: plugins_disco.PluginsRegistry, args: list[str]
                            ) -> NamespaceConfig:
     """Returns parsed command line arguments.
 
@@ -134,6 +135,13 @@ def prepare_and_parse_args(plugins: plugins_disco.PluginsRegistry, args: List[st
         metavar="EAB_HMAC_KEY",
         help="HMAC key for External Account Binding"
     )
+    helpful.add(
+        [None, "run", "certonly", "register"],
+        "--eab-hmac-alg", dest="eab_hmac_alg",
+        metavar="EAB_HMAC_ALG",
+        default=flag_default("eab_hmac_alg"),
+        help="HMAC algorithm for External Account Binding"
+    )
     helpful.add(
         [None, "run", "certonly", "manage", "delete", "certificates",
          "renew", "enhance", "reconfigure"], "--cert-name", dest="certname",
@@ -420,9 +428,9 @@ def prepare_and_parse_args(plugins: plugins_disco.PluginsRegistry, args: List[st
         default=flag_default("random_sleep_on_renew"), dest="random_sleep_on_renew",
         help=argparse.SUPPRESS)
     helpful.add(
-        ["renew", "reconfigure"], "--deploy-hook", action=_DeployHookAction,
+        ["certonly", "renew", "reconfigure", "run"], "--deploy-hook", action=_DeployHookAction,
         help='Command to be run in a shell once for each successfully'
-        ' issued certificate.'
+        ' issued certificate, including on subsequent renewals.'
         ' Unless --disable-hook-validation is used, the command’s first word'
         ' must be the absolute pathname of an executable or one found via the'
         ' PATH environment variable.'
@@ -483,7 +491,7 @@ def prepare_and_parse_args(plugins: plugins_disco.PluginsRegistry, args: List[st
     return helpful.parse_args()
 
 
-def argparse_type(variable: Any) -> Type:
+def argparse_type(variable: Any) -> type:
     """Return our argparse type function for a config variable (default: str)"""
     # pylint: disable=protected-access
     if helpful_parser is not None:
