@@ -1,40 +1,35 @@
 """Certbot command line argument parser"""
 
 import argparse
-import copy
 import functools
-import glob
 import sys
 from typing import Any
-from typing import Dict
 from typing import Iterable
-from typing import List
 from typing import Optional
 from typing import Union
 
 import configargparse
+from cryptography import x509
 
 from certbot import crypto_util
 from certbot import errors
 from certbot import util
 from certbot._internal import constants
 from certbot._internal import hooks
-from certbot._internal.cli.cli_constants import ARGPARSE_PARAMS_TO_REMOVE
+from certbot._internal import san
 from certbot._internal.cli.cli_constants import COMMAND_OVERVIEW
-from certbot._internal.cli.cli_constants import EXIT_ACTIONS
 from certbot._internal.cli.cli_constants import HELP_AND_VERSION_USAGE
 from certbot._internal.cli.cli_constants import SHORT_USAGE
-from certbot._internal.cli.cli_constants import ZERO_ARG_ACTIONS
-from certbot._internal.cli.cli_utils import _Default
-from certbot._internal.cli.cli_utils import add_domains
 from certbot._internal.cli.cli_utils import CustomHelpFormatter
 from certbot._internal.cli.cli_utils import flag_default
 from certbot._internal.cli.cli_utils import HelpfulArgumentGroup
+from certbot._internal.cli.cli_utils import set_test_server_options
 from certbot._internal.cli.verb_help import VERB_HELP
 from certbot._internal.cli.verb_help import VERB_HELP_MAP
 from certbot._internal.display import obj as display_obj
 from certbot._internal.plugins import disco
-from certbot.compat import os
+from certbot.configuration import ArgumentSource
+from certbot.configuration import NamespaceConfig
 
 
 class HelpfulArgumentParser:
@@ -45,8 +40,7 @@ class HelpfulArgumentParser:
     'certbot --help security' for security options.
 
     """
-    def __init__(self, args: List[str], plugins: Iterable[str],
-                 detect_defaults: bool = False) -> None:
+    def __init__(self, args: list[str], plugins: Iterable[str]) -> None:
         from certbot._internal import main
         self.VERBS = {
             "auth": main.certonly,
@@ -62,7 +56,6 @@ class HelpfulArgumentParser:
             "revoke": main.revoke,
             "rollback": main.rollback,
             "everything": main.run,
-            "update_symlinks": main.update_symlinks,
             "certificates": main.certificates,
             "delete": main.delete,
             "enhance": main.enhance,
@@ -72,14 +65,15 @@ class HelpfulArgumentParser:
         # Get notification function for printing
         self.notify = display_obj.NoninteractiveDisplay(sys.stdout).notification
 
+        self.actions: list[configargparse.Action] = []
+
         # List of topics for which additional help can be provided
-        HELP_TOPICS: List[Optional[str]] = ["all", "security", "paths", "automation", "testing"]
+        HELP_TOPICS: list[Optional[str]] = ["all", "security", "paths", "automation", "testing"]
         HELP_TOPICS += list(self.VERBS) + self.COMMANDS_TOPICS + ["manage"]
 
-        plugin_names: List[Optional[str]] = list(plugins)
-        self.help_topics: List[Optional[str]] = HELP_TOPICS + plugin_names + [None]
+        plugin_names: list[Optional[str]] = list(plugins)
+        self.help_topics: list[Optional[str]] = HELP_TOPICS + plugin_names + [None]
 
-        self.detect_defaults = detect_defaults
         self.args = args
 
         if self.args and self.args[0] == 'help':
@@ -99,9 +93,7 @@ class HelpfulArgumentParser:
         self.visible_topics = self.determine_help_topics(self.help_arg)
 
         # elements are added by .add_group()
-        self.groups: Dict[str, argparse._ArgumentGroup] = {}
-        # elements are added by .parse_args()
-        self.defaults: Dict[str, Any] = {}
+        self.groups: dict[str, argparse._ArgumentGroup] = {}
 
         self.parser = configargparse.ArgParser(
             prog="certbot",
@@ -166,135 +158,201 @@ class HelpfulArgumentParser:
 
         return usage
 
-    def remove_config_file_domains_for_renewal(self, parsed_args: argparse.Namespace) -> None:
+    def remove_config_file_domains_for_renewal(self, config: NamespaceConfig) -> None:
         """Make "certbot renew" safe if domains are set in cli.ini."""
         # Works around https://github.com/certbot/certbot/issues/4096
-        if self.verb == "renew":
-            for source, flags in self.parser._source_to_settings.items(): # pylint: disable=protected-access
-                if source.startswith("config_file") and "domains" in flags:
-                    parsed_args.domains = _Default() if self.detect_defaults else []
+        assert config.argument_sources is not None
+        if (config.argument_sources['domains'] == ArgumentSource.CONFIG_FILE and
+                self.verb == "renew"):
+            config.domains = []
 
-    def parse_args(self) -> argparse.Namespace:
+    def _build_sources_dict(self) -> dict[str, ArgumentSource]:
+        # ConfigArgparse's get_source_to_settings_dict doesn't actually create
+        # default entries for each argument with a default value, omitting many
+        # args we'd otherwise care about. So in general, unless an argument was
+        # specified in a config file/environment variable/command line arg,
+        # consider it as having a "default" value
+        result = { action.dest: ArgumentSource.DEFAULT for action in self.actions }
+
+        source_to_settings_dict: dict[str, dict[str, tuple[configargparse.Action, str]]]
+        source_to_settings_dict = self.parser.get_source_to_settings_dict()
+
+        # We'll process the sources dict in order of each source's "priority",
+        # i.e. the order in which ConfigArgparse ultimately sets argument
+        # values:
+        #   1. defaults (`result` already has everything marked as such)
+        #   2. config files
+        #   3. env vars (shouldn't be any)
+        #   4. command line
+
+        def update_result(settings_dict: dict[str, tuple[configargparse.Action, str]],
+                          source: ArgumentSource) -> None:
+            actions = [self._find_action_for_arg(arg) if action is None else action
+                       for arg, (action, _) in settings_dict.items()]
+            result.update({ action.dest: source for action in actions })
+
+        # config file sources look like "config_file|<name of file>"
+        for source_key in source_to_settings_dict:
+            if source_key.startswith('config_file'):
+                update_result(source_to_settings_dict[source_key], ArgumentSource.CONFIG_FILE)
+
+        update_result(source_to_settings_dict.get('env_var', {}), ArgumentSource.ENV_VAR)
+
+        # The command line settings dict is weird, so handle it separately
+        if 'command_line' in source_to_settings_dict:
+            settings_dict: dict[str, tuple[None, list[str]]]
+            settings_dict = source_to_settings_dict['command_line'] # type: ignore
+            (_, unprocessed_args) = settings_dict['']
+            args = []
+            for arg in unprocessed_args:
+                # ignore non-arguments
+                if not arg.startswith('-'):
+                    continue
+
+                # special case for config file argument, which we don't have an action for
+                if arg in ['-c', '--config']:
+                    result['config_dir'] = ArgumentSource.COMMAND_LINE
+                    continue
+
+                if '=' in arg:
+                    arg = arg.split('=')[0]
+                elif ' ' in arg:
+                    arg = arg.split(' ')[0]
+
+                if arg.startswith('--'):
+                    args.append(arg)
+                # for short args (ones that start with a single hyphen), handle
+                # the case of multiple short args together, e.g. "-tvm"
+                else:
+                    for short_arg in arg[1:]:
+                        args.append(f"-{short_arg}")
+
+            for arg in args:
+                # find the action corresponding to this arg
+                action = self._find_action_for_arg(arg)
+                result[action.dest] = ArgumentSource.COMMAND_LINE
+
+        return result
+
+    def _find_action_for_arg(self, arg: str) -> configargparse.Action:
+        # Finds a configargparse Action which matches the given arg, where arg
+        # can either be preceded by hyphens (as on the command line) or not (as
+        # in config files)
+
+        # if the argument doesn't have leading hyphens, prefix it so it can be
+        # compared directly w/ action option strings
+        if arg[0] != '-':
+            arg = '--' + arg
+
+        # first, check for exact matches
+        for action in self.actions:
+            if arg in action.option_strings:
+                return action
+
+        # now check for abbreviated (i.e. prefix) matches
+        for action in self.actions:
+            for option_string in action.option_strings:
+                if option_string.startswith(arg):
+                    return action
+
+        raise AssertionError(f"Action corresponding to argument {arg} is None")
+
+    def parse_args(self) -> NamespaceConfig:
         """Parses command line arguments and returns the result.
 
         :returns: parsed command line arguments
-        :rtype: argparse.Namespace
+        :rtype: configuration.NamespaceConfig
 
         """
         parsed_args = self.parser.parse_args(self.args)
         parsed_args.func = self.VERBS[self.verb]
         parsed_args.verb = self.verb
+        config = NamespaceConfig(parsed_args)
+        config.set_argument_sources(self._build_sources_dict())
 
-        self.remove_config_file_domains_for_renewal(parsed_args)
-
-        if self.detect_defaults:
-            return parsed_args
-
-        self.defaults = {key: copy.deepcopy(self.parser.get_default(key))
-                             for key in vars(parsed_args)}
+        self.remove_config_file_domains_for_renewal(config)
 
         # Do any post-parsing homework here
 
         if self.verb == "renew":
-            if parsed_args.force_interactive:
+            if config.force_interactive:
                 raise errors.Error(
                     "{0} cannot be used with renew".format(
                         constants.FORCE_INTERACTIVE_FLAG))
-            parsed_args.noninteractive_mode = True
+            config.noninteractive_mode = True
 
-        if parsed_args.force_interactive and parsed_args.noninteractive_mode:
+        if config.force_interactive and config.noninteractive_mode:
             raise errors.Error(
                 "Flag for non-interactive mode and {0} conflict".format(
                     constants.FORCE_INTERACTIVE_FLAG))
 
-        if parsed_args.staging or parsed_args.dry_run:
-            self.set_test_server(parsed_args)
+        if config.staging or config.dry_run:
+            self.set_test_server(config)
 
-        if parsed_args.csr:
-            self.handle_csr(parsed_args)
+        if config.csr:
+            self.handle_csr(config)
 
-        if parsed_args.must_staple:
-            parsed_args.staple = True
+        if config.must_staple and not config.staple:
+            config.staple = True
 
-        if parsed_args.validate_hooks:
-            hooks.validate_hooks(parsed_args)
+        if config.validate_hooks:
+            hooks.validate_hooks(config)
 
-        if parsed_args.allow_subset_of_names:
-            if any(util.is_wildcard_domain(d) for d in parsed_args.domains):
+        if config.allow_subset_of_names:
+            if any(d.is_wildcard() for d in config.domains):
                 raise errors.Error("Using --allow-subset-of-names with a"
                                    " wildcard domain is not supported.")
 
-        if parsed_args.hsts and parsed_args.auto_hsts:
+        if config.hsts and config.auto_hsts:
             raise errors.Error(
                 "Parameters --hsts and --auto-hsts cannot be used simultaneously.")
 
-        if isinstance(parsed_args.key_type, list) and len(parsed_args.key_type) > 1:
+        if isinstance(config.key_type, list) and len(config.key_type) > 1:
             raise errors.Error(
                 "Only *one* --key-type type may be provided at this time.")
 
-        return parsed_args
+        return config
 
-    def set_test_server(self, parsed_args: argparse.Namespace) -> None:
-        """We have --staging/--dry-run; perform sanity check and set config.server"""
+    def set_test_server(self, config: NamespaceConfig) -> None:
+        """Updates server, break_my_certs, staging, tos, and
+        register_unsafely_without_email in config as necessary to prepare
+        to use the test server."""
+        return set_test_server_options(self.verb, config)
 
-        # Flag combinations should produce these results:
-        #                             | --staging      | --dry-run   |
-        # ------------------------------------------------------------
-        # | --server acme-v02         | Use staging    | Use staging |
-        # | --server acme-staging-v02 | Use staging    | Use staging |
-        # | --server <other>          | Conflict error | Use <other> |
-
-        default_servers = (flag_default("server"), constants.STAGING_URI)
-
-        if parsed_args.staging and parsed_args.server not in default_servers:
-            raise errors.Error("--server value conflicts with --staging")
-
-        if parsed_args.server in default_servers:
-            parsed_args.server = constants.STAGING_URI
-
-        if parsed_args.dry_run:
-            if self.verb not in ["certonly", "renew"]:
-                raise errors.Error("--dry-run currently only works with the "
-                                   "'certonly' or 'renew' subcommands (%r)" % self.verb)
-            parsed_args.break_my_certs = parsed_args.staging = True
-            if glob.glob(os.path.join(parsed_args.config_dir, constants.ACCOUNTS_DIR, "*")):
-                # The user has a prod account, but might not have a staging
-                # one; we don't want to start trying to perform interactive registration
-                parsed_args.tos = True
-                parsed_args.register_unsafely_without_email = True
-
-    def handle_csr(self, parsed_args: argparse.Namespace) -> None:
+    def handle_csr(self, config: NamespaceConfig) -> None:
         """Process a --csr flag."""
-        if parsed_args.verb != "certonly":
+        if config.verb != "certonly":
             raise errors.Error("Currently, a CSR file may only be specified "
                                "when obtaining a new or replacement "
                                "via the certonly command. Please try the "
                                "certonly command instead.")
-        if parsed_args.allow_subset_of_names:
+        if config.allow_subset_of_names:
             raise errors.Error("--allow-subset-of-names cannot be used with --csr")
 
-        csrfile, contents = parsed_args.csr[0:2]
-        typ, csr, domains = crypto_util.import_csr_file(csrfile, contents)
+        csrfile, contents = config.csr[0:2]
+        typ, util_csr, _ = crypto_util.import_csr_file(csrfile, contents)
+        x509_req = x509.load_pem_x509_csr(util_csr.data)
+        domains, _ = san.from_x509(x509_req.subject, x509_req.extensions)
 
-        # This is not necessary for webroot to work, however,
-        # obtain_certificate_from_csr requires parsed_args.domains to be set
-        for domain in domains:
-            add_domains(parsed_args, domain)
+        # The SANs from the CSR are added to the domains from command line flags as this config
+        # setting is where main.certonly gets the list of identifiers to request.
+        config.domains.extend(domains)
 
         if not domains:
             # TODO: add CN to domains instead:
             raise errors.Error(
                 "Unfortunately, your CSR %s needs to have a SubjectAltName for every domain"
-                % parsed_args.csr[0])
+                % config.csr[0])
 
-        parsed_args.actual_csr = (csr, typ)
+        config.actual_csr = (util_csr, typ)
 
-        csr_domains = {d.lower() for d in domains}
-        config_domains = set(parsed_args.domains)
-        if csr_domains != config_domains:
+        # Check that the original values for --domain set by the user were
+        # a subset of the domains listed in the CSR.
+        if set(config.domains) != set(domains):
             raise errors.ConfigurationError(
-                "Inconsistent domain requests:\nFrom the CSR: {0}\nFrom command line/config: {1}"
-                .format(", ".join(csr_domains), ", ".join(config_domains)))
+                "Inconsistent requests:\nFrom the CSR: {0}\nFrom command line/config: {1}"
+                .format(san.display(domains),
+                        san.display(config.domains)))
 
 
     def determine_verb(self) -> None:
@@ -343,7 +401,7 @@ class HelpfulArgumentParser:
             pass
         return True
 
-    def add(self, topics: Optional[Union[List[Optional[str]], str]], *args: Any,
+    def add(self, topics: Optional[Union[list[Optional[str]], str]], *args: Any,
             **kwargs: Any) -> None:
         """Add a new command line argument.
 
@@ -356,6 +414,10 @@ class HelpfulArgumentParser:
         :param dict **kwargs: various argparse settings for this argument
 
         """
+        self.actions.append(self._add(topics, *args, **kwargs))
+
+    def _add(self, topics: Optional[Union[list[Optional[str]], str]], *args: Any,
+            **kwargs: Any) -> configargparse.Action:
         action = kwargs.get("action")
         if action is util.DeprecatedArgumentAction:
             # If the argument is deprecated through
@@ -366,8 +428,7 @@ class HelpfulArgumentParser:
             # handling default detection since these actions aren't needed and
             # can cause bugs like
             # https://github.com/certbot/certbot/issues/8495.
-            self.parser.add_argument(*args, **kwargs)
-            return
+            return self.parser.add_argument(*args, **kwargs)
 
         if isinstance(topics, list):
             # if this flag can be listed in multiple sections, try to pick the one
@@ -376,40 +437,15 @@ class HelpfulArgumentParser:
         else:
             topic = topics  # there's only one
 
-        if self.detect_defaults:
-            kwargs = self.modify_kwargs_for_default_detection(**kwargs)
-
         if not isinstance(topic, bool) and self.visible_topics[topic]:
             if topic in self.groups:
                 group = self.groups[topic]
-                group.add_argument(*args, **kwargs)
+                return group.add_argument(*args, **kwargs)
             else:
-                self.parser.add_argument(*args, **kwargs)
+                return self.parser.add_argument(*args, **kwargs)
         else:
             kwargs["help"] = argparse.SUPPRESS
-            self.parser.add_argument(*args, **kwargs)
-
-    def modify_kwargs_for_default_detection(self, **kwargs: Any) -> Dict[str, Any]:
-        """Modify an arg so we can check if it was set by the user.
-
-        Changes the parameters given to argparse when adding an argument
-        so we can properly detect if the value was set by the user.
-
-        :param dict kwargs: various argparse settings for this argument
-
-        :returns: a modified versions of kwargs
-        :rtype: dict
-
-        """
-        action = kwargs.get("action", None)
-        if action not in EXIT_ACTIONS:
-            kwargs["action"] = ("store_true" if action in ZERO_ARG_ACTIONS else
-                                "store")
-            kwargs["default"] = _Default()
-            for param in ARGPARSE_PARAMS_TO_REMOVE:
-                kwargs.pop(param, None)
-
-        return kwargs
+            return self.parser.add_argument(*args, **kwargs)
 
     def add_deprecated_argument(self, argument_name: str, num_args: int) -> None:
         """Adds a deprecated argument with the name argument_name.
@@ -475,7 +511,7 @@ class HelpfulArgumentParser:
             plugin_ep.plugin_cls.inject_parser_options(parser_or_group, name)
 
     def determine_help_topics(self, chosen_topic: Union[str, bool]
-                              ) -> Dict[Optional[str], bool]:
+                              ) -> dict[Optional[str], bool]:
         """
 
         The user may have requested help on a topic, return a dict of which
