@@ -2,6 +2,7 @@
 import argparse
 import atexit
 import errno
+import itertools
 import logging
 import platform
 import re
@@ -10,21 +11,17 @@ import subprocess
 import sys
 from typing import Any
 from typing import Callable
-from typing import Dict
 from typing import IO
-from typing import List
 from typing import NamedTuple
 from typing import Optional
-from typing import Set
-from typing import Tuple
 from typing import Union
-import warnings
 
 import configargparse
 
 from certbot import errors
 from certbot._internal import constants
 from certbot._internal import lock
+from certbot._internal import san
 from certbot.compat import filesystem
 from certbot.compat import os
 
@@ -49,6 +46,79 @@ class CSR(NamedTuple):
     form: str
 
 
+class LooseVersion:
+    """A version with loose rules, i.e. any given string is a valid version number.
+
+    but regular comparison is not supported. Instead, the `try_risky_comparison` method is
+    provided, which may return an error if two LooseVersions are 'incomparible'.
+    For example when integer and string version components are present in the same position.
+
+    Differences with old distutils.version.LooseVersion:
+    (https://github.com/python/cpython/blob/v3.10.0/Lib/distutils/version.py#L269)
+    Most version comparisons should give the same result. However, if a version has multiple
+    trailing zeroes, not all of them are used in the comparison. This ensure that, for example,
+    "2.0" and "2.0.0" are equal.
+    """
+
+    def __init__(self, version_string: str) -> None:
+        """Parses a version string into its components.
+
+        :param str version_string: version string
+        """
+        components: list[Union[int, str]]
+        components = [x for x in _VERSION_COMPONENT_RE.split(version_string)
+                              if x and x != '.']
+        for i, obj in enumerate(components):
+            try:
+                components[i] = int(obj)
+            except ValueError:
+                pass
+
+        self.version_components = components
+
+    def try_risky_comparison(self, other: 'LooseVersion') -> int:
+        """Compares the LooseVersion to another value.
+
+        If the other value is another LooseVersion, the version components are compared. Otherwise,
+        an exception is raised.
+
+        Comparison is performed element-wise. If the version components being compared are of
+        different types, the two versions are considered incompatible. Otherwise, if either of the
+        components is not equal to the other, less or greater is returned based on the comparison's
+        result. In case the two versions are of different lengths, some elements in the longer
+        version have not yet been compared. If these are all equal to zero, the two versions are
+        equal. Otherwise, the longer version is greater.
+
+        If the two versions are incompatible, an exception is raised. Otherwise, the returned
+        integer indicates the result of the comparison. If self == other, 0 is returned.
+        If self > other, 1 is returned. If self < other -1 is returned.
+
+        Examples:
+        Equality:
+        - LooseVersion('1.0').try_risky_comparison(LooseVersion('1.0')) -> 0
+        - LooseVersion('2.0.0a').try_risky_comparison(LooseVersion('2.0.0a')) -> 0
+        Inequality:
+        - LooseVersion('2.0.0').try_risky_comparison(LooseVersion('1.0')) -> 1
+        - LooseVersion('1.0.1').try_risky_comparison(LooseVersion('2.0a')) -> -1
+        Incomparability:
+        - LooseVersion('1a').try_risky_comparison(LooseVersion('1.0')) -> ValueError
+        """
+        try:
+            for self_vc, other_vc in itertools.zip_longest(self.version_components,
+                                                           other.version_components,
+                                                           fillvalue=0):
+                # ensure mypy ignores types here and catch any TypeErrors
+                if self_vc < other_vc:  # type: ignore
+                    return -1
+                elif self_vc > other_vc:  # type: ignore
+                    return 1
+            return 0
+        except TypeError:
+            raise ValueError("Cannot meaningfully compare LooseVersion {} with LooseVersion {} "
+                             "due to comparison of version components with different types."
+                             .format(self.version_components, other.version_components))
+
+
 # ANSI SGR escape codes
 # Formats text as bold or with increased intensity
 ANSI_SGR_BOLD = '\033[1m'
@@ -70,10 +140,10 @@ _INITIAL_PID = os.getpid()
 # the dict are attempted to be cleaned up at program exit. If the
 # program exits before the lock is cleaned up, it is automatically
 # released, but the file isn't deleted.
-_LOCKS: Dict[str, lock.LockFile] = {}
+_LOCKS: dict[str, lock.LockFile] = {}
 _VERSION_COMPONENT_RE = re.compile(r'(\d+ | [a-z]+ | \.)', re.VERBOSE)
 
-def env_no_snap_for_external_calls() -> Dict[str, str]:
+def env_no_snap_for_external_calls() -> dict[str, str]:
     """
     When Certbot is run inside a Snap, certain environment variables
     are modified. But Certbot sometimes calls out to external programs,
@@ -92,13 +162,19 @@ def env_no_snap_for_external_calls() -> Dict[str, str]:
     # Avoid accidentally modifying env
     if 'SNAP' not in env or 'CERTBOT_SNAPPED' not in env:
         return env
+
+    # These environment variables being set when running external programs can cause issues if these
+    # programs also use OpenSSL. See https://github.com/certbot/certbot/issues/10190.
+    env.pop('OPENSSL_FORCE_FIPS_MODE', None)
+    env.pop('OPENSSL_MODULES', None)
+
     for path_name in ('PATH', 'LD_LIBRARY_PATH'):
         if path_name in env:
             env[path_name] = ':'.join(x for x in env[path_name].split(':') if env['SNAP'] not in x)
     return env
 
 
-def run_script(params: List[str], log: Callable[[str], None]=logger.error) -> Tuple[str, str]:
+def run_script(params: list[str], log: Callable[[str], None]=logger.error) -> tuple[str, str]:
     """Run the script with the given params.
 
     :param list params: List of parameters to pass to subprocess.run
@@ -227,16 +303,16 @@ def safe_open(path: str, mode: str = "w", chmod: Optional[int] = None) -> IO:
         if ``None``.
 
     """
-    open_args: Union[Tuple[()], Tuple[int]] = ()
+    open_args: Union[tuple[()], tuple[int]] = ()
     if chmod is not None:
         open_args = (chmod,)
-    fdopen_args: Union[Tuple[()], Tuple[int]] = ()
+    fdopen_args: Union[tuple[()], tuple[int]] = ()
     fd = filesystem.open(path, os.O_CREAT | os.O_EXCL | os.O_RDWR, *open_args)
     return os.fdopen(fd, mode, *fdopen_args)
 
 
 def _unique_file(path: str, filename_pat: Callable[[int], str], count: int,
-                 chmod: int, mode: str) -> Tuple[IO, str]:
+                 chmod: int, mode: str) -> tuple[IO, str]:
     while True:
         current_path = os.path.join(path, filename_pat(count))
         try:
@@ -248,7 +324,7 @@ def _unique_file(path: str, filename_pat: Callable[[int], str], count: int,
         count += 1
 
 
-def unique_file(path: str, chmod: int = 0o777, mode: str = "w") -> Tuple[IO, str]:
+def unique_file(path: str, chmod: int = 0o777, mode: str = "w") -> tuple[IO, str]:
     """Safely finds a unique file.
 
     :param str path: path/filename.ext
@@ -265,7 +341,7 @@ def unique_file(path: str, chmod: int = 0o777, mode: str = "w") -> Tuple[IO, str
 
 
 def unique_lineage_name(path: str, filename: str, chmod: int = 0o644,
-                        mode: str = "w") -> Tuple[IO, str]:
+                        mode: str = "w") -> tuple[IO, str]:
     """Safely finds a unique file using lineage convention.
 
     :param str path: directory path
@@ -301,7 +377,7 @@ def safely_remove(path: str) -> None:
             raise
 
 
-def get_filtered_names(all_names: Set[str]) -> Set[str]:
+def get_filtered_names(all_names: set[str]) -> set[str]:
     """Removes names that aren't considered valid by Let's Encrypt.
 
     :param set all_names: all names found in the configuration
@@ -318,7 +394,7 @@ def get_filtered_names(all_names: Set[str]) -> Set[str]:
             logger.debug('Not suggesting name "%s"', name, exc_info=True)
     return filtered_names
 
-def get_os_info() -> Tuple[str, str]:
+def get_os_info() -> tuple[str, str]:
     """
     Get OS name and version
 
@@ -334,7 +410,11 @@ def get_os_info_ua() -> str:
 
     :returns: os_ua
     :rtype: `str`
+
     """
+    # distro.name returns an empty string if one cannot be determined. see
+    # https://github.com/python-distro/distro/blob/3bd19e61fcb7f8d2bf3d45d9e40d69c92e05d241/src/distro/distro.py#L883
+    os_info = ""
     if _USE_DISTRO:
         os_info = distro.name(pretty=True)
 
@@ -342,7 +422,7 @@ def get_os_info_ua() -> str:
         return " ".join(get_python_os_info(pretty=True))
     return os_info
 
-def get_systemd_os_like() -> List[str]:
+def get_systemd_os_like() -> list[str]:
     """
     Get a list of strings that indicate the distribution likeness to
     other distributions.
@@ -384,7 +464,7 @@ def _normalize_string(orig: str) -> str:
     """
     return orig.replace('"', '').replace("'", "").strip()
 
-def get_python_os_info(pretty: bool = False) -> Tuple[str, str]:
+def get_python_os_info(pretty: bool = False) -> tuple[str, str]:
     """
     Get Operating System type/distribution and major version
     using python platform module
@@ -455,7 +535,7 @@ class DeprecatedArgumentAction(argparse.Action):
     """Action to log a warning when an argument is used."""
     def __call__(self, unused1: Any, unused2: Any, unused3: Any,
                  option_string: Optional[str] = None) -> None:
-        warnings.warn("Use of %s is deprecated." % option_string, DeprecationWarning)
+        logger.warning("Use of %s is deprecated.", option_string)
 
 
 def add_deprecated_argument(add_argument: Callable[..., None], argument_name: str,
@@ -497,7 +577,8 @@ def enforce_le_validity(domain: str) -> str:
 
     """
 
-    domain = enforce_domain_sanity(domain)
+    # Do basic validation on a DNSName
+    domain = san.DNSName(domain).dns_name
     if not re.match("^[A-Za-z0-9.-]*$", domain):
         raise errors.ConfigurationError(
             "{0} contains an invalid character. "
@@ -592,12 +673,12 @@ def is_ipaddress(address: str) -> bool:
         socket.inet_pton(socket.AF_INET, address)
         # If this line runs it was ip address (ipv4)
         return True
-    except socket.error:
+    except OSError:
         # It wasn't an IPv4 address, so try ipv6
         try:
             socket.inet_pton(socket.AF_INET6, address)
             return True
-        except socket.error:
+        except OSError:
             return False
 
 
@@ -639,30 +720,19 @@ def atexit_register(func: Callable, *args: Any, **kwargs: Any) -> None:
     atexit.register(_atexit_call, func, *args, **kwargs)
 
 
-def parse_loose_version(version_string: str) -> List[Union[int, str]]:
+def parse_loose_version(version_string: str) -> list[Union[int, str]]:
     """Parses a version string into its components.
-
     This code and the returned tuple is based on the now deprecated
     distutils.version.LooseVersion class from the Python standard library.
     Two LooseVersion classes and two lists as returned by this function should
     compare in the same way. See
     https://github.com/python/cpython/blob/v3.10.0/Lib/distutils/version.py#L205-L347.
-
     :param str version_string: version string
-
     :returns: list of parsed version string components
     :rtype: list
-
     """
-    components: List[Union[int, str]]
-    components = [x for x in _VERSION_COMPONENT_RE.split(version_string)
-                          if x and x != '.']
-    for i, obj in enumerate(components):
-        try:
-            components[i] = int(obj)
-        except ValueError:
-            pass
-    return components
+    loose_version = LooseVersion(version_string)
+    return loose_version.version_components
 
 
 def _atexit_call(func: Callable, *args: Any, **kwargs: Any) -> None:
